nextflow.enable.dsl = 2

include {checksums} from './file_checks.nf'

params.ped_file = null
params.sample_sheet = null

ch_sample_sheet = Channel.fromPath(params.sample_sheet, checkIfExists: true)
ch_ped_file = Channel.fromPath(params.ped_file, checkIfExists: true)


workflow read_ped_file {
    /*
    Read the given ped file and output a tuple channel of included families and their members:

    {
        family1: [
            {
                family: family1,
                indv: indv1,
                father: indv2,
                mother: indv3,
                sex: 2,
                affected: 2
            },
            {
                ...
            },
        
        family2: [
            ...
        ]
    }
    */

    take:
        ch_ped_file

    main:
        ch_family_info = ch_ped_file
            .splitCsv(sep: '\t')
            .map(
                { line ->
                    [
                        line[0],  // family ID
                        [
                            family: line[0],
                            indv: line[1],
                            father: line[2],
                            mother: line[3],
                            sex: line[4],
                            affected: line[5]
                        ]
                    ]
                }
            )
            .groupTuple()
    
    emit:
        ch_family_info
}

workflow read_sample_sheet {
    /*
    Read in a pipeline sample sheet and output a tuple of included individuals and their listed fastqs:
    {
        indv1: {
            r1: [lane_1_r1.fastq.gz, lane_2_r1.fastq.gz],
            r2: [lane_1_r2.fastq.gz, lane_2_r2.fastq.gz]
        },
        indv2: {
            ...
        }
    }
    */

    take:
        ch_sample_sheet

    main:
        ch_samplesheet_info = ch_sample_sheet
            .splitCsv(sep:'\t', header: true)
            .map(
                { line -> [line.individual_id, line.read_1, line.read_2] }
            )
            .groupTuple()
            .map(
                { record ->
                    [
                        record[0],
                        [
                            indv: record[0],
                            r1: record[1],
                            r2: record[2]
                        ]
                    ]
                }
            )
    
    emit: ch_samplesheet_info
}

workflow {
    checksums(read_sample_sheet(ch_sample_sheet))
}
